/**
 * @file
 * VuoList_VuoMultisample C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoMultisample_H
#define VuoList_VuoMultisample_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoMultisample VuoList_VuoMultisample
 * A list of @ref VuoMultisample elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoMultisample_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoMultisample elements.
 */
typedef const struct VuoList_VuoMultisample_struct { void *l; } * VuoList_VuoMultisample;
/// @}
#endif

/**
 * Creates a new list of @ref VuoMultisample elements.
 */
VuoList_VuoMultisample VuoListCreate_VuoMultisample(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoMultisample to quickly initialize a large list.
 */
VuoList_VuoMultisample VuoListCreateWithCount_VuoMultisample(const unsigned long count, const VuoMultisample value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoMultisample VuoListCopy_VuoMultisample(const VuoList_VuoMultisample list);

/**
 * Returns the @ref VuoMultisample at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoMultisample VuoListGetValue_VuoMultisample(const VuoList_VuoMultisample list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoMultisample *VuoListGetData_VuoMultisample(const VuoList_VuoMultisample list);

/**
 * Changes the @ref VuoMultisample at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoMultisample(const VuoList_VuoMultisample list, const VuoMultisample value, const unsigned long index);

/**
 * Inserts the `VuoMultisample` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoMultisample(const VuoList_VuoMultisample list, const VuoMultisample value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoMultisample(VuoList_VuoMultisample list, const VuoMultisample value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoMultisample(VuoList_VuoMultisample list, const VuoMultisample value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoMultisample(VuoList_VuoMultisample list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoMultisample_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoMultisample(VuoList_VuoMultisample list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoMultisample_areEqual(const VuoList_VuoMultisample a, const VuoList_VuoMultisample b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoMultisample_isLessThan(const VuoList_VuoMultisample a, const VuoList_VuoMultisample b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoMultisample(VuoList_VuoMultisample list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoMultisample(VuoList_VuoMultisample list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoMultisample(VuoList_VuoMultisample list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoMultisample(VuoList_VuoMultisample list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoMultisample(VuoList_VuoMultisample list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoMultisample(VuoList_VuoMultisample list);

/**
 * Removes the `VuoMultisample` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoMultisample(VuoList_VuoMultisample list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoMultisample(const VuoList_VuoMultisample list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoMultisample VuoList_VuoMultisample_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoMultisample_getJson(const VuoList_VuoMultisample value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoMultisample_getSummary(const VuoList_VuoMultisample value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoMultisample VuoList_VuoMultisample_makeFromString(const char *str);
char * VuoList_VuoMultisample_getString(const VuoList_VuoMultisample value);
/// @}

/**
 * @}
 */

#endif
